#ifndef _EXP_
#define _EXP_

#include "../MISC/parameters.h"
#include <math.h>
#ifdef _GRAPHICS_
#include "../EXP/RENDERING/engine.h"

class EXP_Class : public Engine {
#endif
  
#ifndef _GRAPHICS_
  class EXP_Class {
#endif
    
  private:
    /* --------------- FIXED MEMBERS --------------- */
    int  generation;//Count the num. of generation
    int  from_gen;
    int  to_gen;
    int  num_core;
    int  genotype; //Count the num. of genotype (indivituals) population size
    int  eval; //Count the num. of evaluation
    int  num_collision;
    int  rank,counter;
    double collision_ratio;
    int  iter; //Count the num. of iterations
    int remaining_iter;
    bool evolution;
    bool viewing;
    bool re_evaluation;
    bool touched, check, transported;
    vector <double> evaluatoinal_min_fitness;
    double generational_avg_fitness;
    double generational_max_fitness;
    double generational_min_fitness;
    vector <double> FINAL_FITNESS;
    vector < vector <double> > REEVAL_FITNESS;
    vector < vector <double> > DIST_FITNESS;
    vector < vector <double> > pos;
    vector < vector <double> > rot;
    vector <double> centre;
    vector <double> sensors_inputs,motor_outputs;
  
    double current_dist;
    double start_distance;
    double final_distance;
    double init_agent_dist_to_object;
    double init_object_dist_to_nest;
    double dist_achieved;
    double agents_closeness_to_object;
    double object_closeness_to_nest;
    double AGGREGATE_FITNESS;
    vector <double> fitnesses_mult;
    double internal_fitness;
    double ir_sum;
    double right_motor_speed;
    double left_motor_speed;
    double frond_IR_readings;
    /* --------------------------------------------- */

  vector <double> ir_readings;
  vector <double> camera_sector_readings;
  vector <double> bearing_sector_reading;
  vector <double> mouse_reading;

 public:

  /* --------------- FIXED FUNCTIONS --------------- */
  EXP_Class (const char *run_name, bool _evolution, bool _viewing, bool _re_evaluation, unsigned long _seed, int _from_gen, int _to_gen , int _num_cores);
  ~EXP_Class( );
  typedef struct {
                     vector <double> inputs;
                     vector <double> outputs;
                     vector < vector <double> > fitness;
   } Interface;
 vector <Interface> agent_interface;
  double max_distance;
  int success;
  Parameters *param;
  double find_distance(const vector <double> &_pos1, const vector <double> &_pos2);
  void init_local_variables( void );
  void init_evolutionary_run( void );
  void init_genotype_loop( );
  void init_evaluations_loop( void );
  void init_single_evaluation( void );
  void set_agent_position( void );
  void from_genome_to_controllers( const char *str_source, const int which_genotype );
  void adv ( void );
  void assign_fitness_to_genotype( void );

  void finalise_single_evaluation( void );
  void finalise_evaluations_loop( void );
  void finalise_genotypes_loop( void );
  void finalise_evolutionary_run( void );
  
  bool stop_iterations_loop( void );
  bool stop_evaluations_loop( void ); 
  bool stop_genotypes_loop( void ); 
  bool stop_generations_loop( void );
  
  void update_sensors( void );
  void update_controllers( void );
  void update_Actuators( void );
  void update_world( void );
  void manage_collisions ( void );
  void compute_internal_fitness();
  void compute_fitness ( void );
  void dump_reevaluation_results ( const char *locationOfFileTodump, const char *fileName, int generation );
  void dump_reevaluation_raw_data( const char *locationOfFileTodump, const char *fileName, int generation );
  void dump_reevaluation_distance( const char *locationOfFileTodump, const char *fileName, int generation );
  void dump_statistics              ( const char *locationOfFileTodump, const char *fileName, int generation );
    /* ----------------------------------------------- */
  
};
#endif
